import octofuss
import os, os.path
from subprocess import Popen, PIPE
import octofussd.model as om
from sqlobject.sqlbuilder import Select

class ServerInfo(octofuss.Tree):
    keyword_doc = {
        "organization": "Organization configured in octofussd",
        "kernel_version": "Current kernel version",
        "ram_mbyte": "Total ram size in megabytes",
        "last_seen_clients": "Last client connected"
    }

    def __init__(self, name = None, **kw):
        super(ServerInfo, self).__init__(name, doc="Server information")
        self.conf = kw.get("conf", None)
        self.data = {}
        if self.conf:
            if self.conf.has_option("General", "organization"):
                self.data['organization'] = self.conf.get("General", "organization")
            else:
                self.data['organization'] = "NOT CONFIGURED"
                         
        self.data['kernel_version'] = Popen(["uname", "-r"], stdout=PIPE).communicate()[0].strip()

        self.data['ram_mbyte'] = os.popen("free -m | head -2 | tail -1 | awk '{print $2;}'").read().strip()
        query = "select hostname,timestamp from client_activity group by hostname order by timestamp desc limit 10"
        ls_data = om.ClientActivity._connection.queryAll(query)
        self.data['last_seen_clients'] = [(x[0] and x[0] or "Unknown", x[1]) for x in ls_data]
        
    def lhas(self, path):
        if len(path) == 0: return True
        if len(path) > 1: return False
        return self.data.has_key(path[0])

    def lget(self, path):
        if len(path) == 0: return self.data
        if len(path) > 1: return super(ServerInfo, self).lget(path)
        return self.data[path[0]]

    def llist(self, path):
        if len(path) == 0: return self.data.keys()
        return []

    def ldoc(self, path):
        if len(path) == 0: return super(ServerInfo, self).ldoc(path)
        if len(path) > 1: return None
        return self.keyword_doc.get(path[0], None)


def init(**kw):
    conf = kw.get("conf", None)
    canMock = conf and conf.getboolean("General", "allowMockup")
    forceMock = conf and conf.getboolean("General", "forceMockup")
    
    yield dict(
        tree = ServerInfo("serverinfo", conf=conf),
        root = "/"
        )
