import octofuss
import os, os.path
from gettext import gettext as _

possible_services = {
    'dansguardian' : _("Content filter"),
    'nscd' : _("Name service cache"),
    'squid' : _("Web Proxy/Cache"),
    'firewall' : _("Firewall"),
    'nfs-kernel-server' : _("Network File System - Server"),
    'nfs-common' : _("Network File System - Common stack")
    }

# Plugin implementation
class Services(octofuss.Tree):
    def __init__(self, name = None):
        super(Services, self).__init__(name, doc="""Service restart facility

This node lists the services in the system that can be restarted.

Set a service to any value to restart it.""")
        self.script_dir = '/etc/init.d'

    def _enabled(self, name):
        return name in possible_services and os.path.isfile(os.path.join(self.script_dir, name))

    def lhas(self, path):
        if len(path) == 0: return True
        if len(path) > 1: return False
        return self._enabled(path[0])

    def lget(self, path):
        if len(path) == 0:
            return dict([i for i in possible_services.iteritems() if self._enabled(i[0])])
        if len(path) > 1: return None
        if not self._enabled(path[0]): return None
        return possible_services[path[0]]
            
    def llist(self, path):
        if len(path) != 0: return []
        return [x for x in possible_services.keys() if self._enabled(x)]

    def lset(self, path, value):
        if len(path) != 1: return super(Services, self).lset(path, value)
        if not self._enabled(path[0]): return None
        return os.system("%s restart" % os.path.join(self.script_dir, path[0]))

    def ldoc(self, path):
        if len(path) == 0: return super(Services, self).ldoc(path)
        if len(path) > 1: return None
        return possible_services.get(path[0], None)
            

def init(**kw):
    yield dict(
            tree = Services("services"),
            root = "/"
            )
