# -*- coding: UTF-8 -*-
import os, os.path
import octofuss
from octofussd.model import Users as UserLog
import actions as act
from urllib import quote_plus, unquote_plus
import smbpasswd

userDB = None
tree = None

_ = lambda x:x


class UserList(octofuss.DynamicView):
    def __init__(self, name, doc="List of users in the system"):
        super(UserList, self).__init__(name, doc)
        #self.specials[":search:"] = UserSearch()

    def users(self):
        """
        Generate a sequence of hosts
        """
        return userDB.list()

    def elements(self):
        return self.users()

    def element(self, name):
        return userDB.user(unquote_plus(name))

    def new_element(self, name, value):
        # Ensure that name is the quoted version of the uid
        if value == "" or value == None: value = {}
        if "uid" not in value:
            value["uid"] = unquote_plus(name)
        elif quote_plus(value["uid"]) != name:
            raise ValueError, "The user name in the tree (%s) should match the escaped uid (%s)" % (name, quote_plus(value["uid"]))
        u = userDB.new_user(value)
        return u.lget([])

    def delete_element(self, name):
        userDB.delete(unquote_plus(name))


class MemberList(octofuss.DynamicView):
    def __init__(self, name, groupName):
        super(MemberList, self).__init__(name, doc="Members of group " + groupName)
        self.groupName = groupName
        #self.specials[":search:"] = UserSearch(baseSearch=["group:"+groupName])

    def users(self):
        """
        Generate a sequence of groups
        """
        return userDB.list(self.groupName)

    def elements(self):
        return self.users()

    def element(self, name):
        """
        Return the user "name" but only if it is a member of this group
        """
        return userDB.user_in_group(unquote_plus(name), self.groupName)

    def new_element(self, name, value):
        user, group = userDB.add_user_to_group(unquote_plus(name), self.groupName)
        return user.lget([])

    def delete_element(self, name):
        userDB.delete_user_from_group(unquote_plus(name), self.groupName)

class UserGroupList(octofuss.DynamicView):
    def __init__(self, name, userName):
        super(UserGroupList, self).__init__(name, doc="Groups of user " + userName)
        self.userName = userName

    def groups(self):
        """
        Generate a sequence of groups
        """
        return userDB.groups_of_user(self.userName)

    def elements(self):
        return self.groups()

    def element(self, name):
        """
        Return the group "name" but only if this user is a member of it
        """
        return userDB.group_in_user(self.userName, unquote_plus(name))

    def new_element(self, name, value):
        user, group = userDB.add_user_to_group(self.userName, unquote_plus(name))
        return group.lget([])

    def delete_element(self, name):
        userDB.delete_user_from_group(self.userName, unquote_plus(name))

class GroupList(octofuss.DynamicView):
    def __init__(self, name, doc="List of groups in the system"):
        super(GroupList, self).__init__(name, doc)

    def groups(self):
        """
        Generate a sequence of hosts
        """
        return userDB.listGroups()

    def elements(self):
        return self.groups()

    def element(self, name):
        return userDB.group(unquote_plus(name))

    def new_element(self, name, value):
        # Ensure that name is the quoted version of the cn
        if value == "" or value == None: value = {}
        if "cn" not in value:
            value["cn"] = unquote_plus(name)
        elif quote_plus(value["cn"]) != name:
            raise ValueError, "The group name in the tree (%s) should match the escaped cn (%s)" % (name, quote_plus(value["cn"]))

        group = userDB.new_group(value)
        return group.lget([])

    def delete_element(self, name):
        userDB.delete_group(unquote_plus(name))

class UserDB(object):
    def __init__(self):
        pass

    def _want_user(self, user):
        """
        If `user` is a string, convert it to a User object
        """
        if isinstance(user, str) or isinstance(user, unicode):
            name = user
            user = self.user(user)
            if user == None:
                raise KeyError, "User %s does not exist" % name
        return user
    
    def _want_group(self, group):
        """
        If `group` is a string, convert it to a Group object
        """
        if isinstance(group, str) or isinstance(group, unicode):
            name = group
            group = self.group(group)
            if group == None:
                raise KeyError, "Group %s does not exist" % name
        return group

    def _fillInNewUser(self, value):
        """
        Fill in 'value' with missing data, to make it a good record for
        creating a new user
        """
        # Check that it doesn't already exist
        user = value["uid"].encode("UTF-8", "replace")
        if self.user(user) != None:
            raise ValueError, "User %s already exists" % user

        if "primaryGroup" in value:
            primaryGroup = self.group(value["primaryGroup"])
            if primaryGroup == None:
                raise ValueError, "Group %s does not exist" % value["primaryGroup"]
            else:
                gidNumber = int(primaryGroup.get("gidNumber"))
        else:
            primaryGroup = None
            gidNumber = self.defaultGidNumber

        # Start with a default template, that later we fill in.
        # We do not edit 'value' in order to avoid bringing in unwanted fields

        #default = dict(
        #   # InetOrgPerson
        #   sn = [],
        #   mail = [],
        #   displayName = [],
        #   mobile = [],
        #   uid = [user],
        #   # ShadowAccount
        #   description = [],
        #   shadowExpire = [],
        uidNumber = self._get_free_uidnumber()
        data = {
            "uid": [user],
            "objectClass": ["top", "person", "organizationalPerson", "inetOrgPerson", "posixAccount", "shadowAccount", "sambaSamAccount" ],
            "sn": [user],
            "cn": [user],
            "displayName": [user],
            "givenName": [user],
            "uidNumber": [str(uidNumber)],
            "gidNumber": [str(gidNumber)],
            "homeDirectory": ["/home/%s" % user],
            "loginShell": ["/bin/bash"],
            "gecos": [user],
            "sambaSID": [self._getUserSID(uidNumber)],
            "sambaPwdLastSet": ["0"],
            "sambaLogonTime": ["0"],
            "sambaLogoffTime": ["2147483647"],
            "sambaKickoffTime": ["2147483647"],
            "sambaPwdCanChange": ["0"],
            "sambaPwdMustChange": ["2147483647"],
            "sambaAcctFlags": ["[UX]"],
            "sambaLMPassword": ["XXX"],
            "sambaPrimaryGroupSID": [self._getUserSID(gidNumber)],
            "sambaNTPassword": ["XXX"],
            "sambaLogonScript": ["%s.cmd" % user],
            "sambaHomeDrive": ["H:"],
        }

        # Handle the various user-settable items in "value"
        for attr in ["gecos", "loginShell", "homeDirectory", "password"]:
            if attr in value:
                data[attr] = [value[attr].encode("UTF-8", "replace")]

        return data

    def _fillInNewGroup(self, value):
        """
        Fill in 'value' with missing data, to make it a good record for
        creating a new user
        """
        # Check that it doesn't already exist
        name = value["cn"].encode("UTF-8", "replace")
        if self.group(name) != None:
            raise ValueError, "Group %s already exists" % name

        gidNumber = self._get_free_gidnumber()

        # Start with a default template, that later we fill in.
        # We do not edit 'value' in order to avoid bringing in unwanted fields

        data = {
            "objectClass": ["top", "posixGroup", "sambaGroupMapping"],
            "sambaGroupType": ["2"],    # 2 is "domain group" (no idea about other numbers)
            "displayName": [name],
            "cn": [name],
            "memberUid": [],
            "sambaSID": [self._getUserSID(gidNumber)],
            "gidNumber": [str(gidNumber)],
#           "description": Netbios Domain Users
        }

        # Handle the various user-settable items in "value"
        for attr in ["description"]:
            if attr in value:
                data[attr] = [value[attr].encode("UTF-8", "replace")]

        return data

def _getfirstor(list, default):
    if len(list) > 0: return list[0]
    return default

class User(octofuss.PyTree):
    def __init__(self, data):
        doc = "User " + data.uid[0]
        if data.gecos: doc += " (" + data.gecos[0] + ")"
        super(User, self).__init__(quote_plus(data.uid[0]), doc=doc)
        self.data = data
        self.groups = UserGroupList("groups", data.uid[0])
        self.__dict__[":actions:"] = act.UserActions(":actions:", self)
        
    def _get(self):
        return dict(
            uid = self.uid,
            uidNumber = self.uidNumber,
            gidNumber = self.gidNumber,
            gecos = self.gecos,
            loginShell = self.loginShell,
            homeDirectory = self.homeDirectory,
            controllerGroup = self.controllerGroup,
            sambaSID = self.sambaSID,
            sambaPrimaryGroupSID = self.sambaPrimaryGroupSID,
        )

    def setGecos(self, value):
        self.data.gecos = [value]
        self.commit()

    def setShell(self, value):
        self.data.loginShell = [value]
        self.commit()

    def setGidNumber(self, value):
        self.data.gidNumber = [value]
        self.commit()
        
    def setHomeDirectory(self, value):
        value = os.path.abspath(value)
        if not value.startswith("/home/"):
            raise ValueError, "The home directory must be somewhere inside /home"
        octofuss.hooks.invoke("move_home_dir", self.data.homeDirectory[0], value)
        self.data.homeDirectory = [value]
        self.commit()

    def setPassword(self, value):
        self.data.sambaLMPassword = smbpasswd.lmhash(value)
        self.data.sambaNTPassword = smbpasswd.nthash(value)
        self.data.userPassword = [value]
        self.commit()

    uid = property(lambda x: x.data.uid[0], doc="User name")
    uidNumber = property(lambda x: x.data.uidNumber[0], doc="Numeric user ID")
    gidNumber = property(lambda x: x.data.gidNumber[0], setGidNumber, doc="Numeric ID of primary group")
    gecos = property(lambda x: _getfirstor(x.data.gecos, ""), setGecos, doc="Full name")
    loginShell = property(lambda x: _getfirstor(x.data.loginShell, ""), setShell, doc="Login shell")
    homeDirectory = property(lambda x: _getfirstor(x.data.homeDirectory, ""), setHomeDirectory, doc="Home directory")
    password = property(lambda x: "hidden", setPassword, doc="""Password

The password can be set, but cannot be read""")
    sambaSID = property(lambda x: _getfirstor(x.data.sambaSID, None), doc="Samba SID")
    sambaPrimaryGroupSID = property(lambda x: _getfirstor(x.data.sambaPrimaryGroupSID, None), doc="Samba SID of primary group")
#   "sn": [user],
#   "cn": [user],
#   "displayName": [user],
#   "givenName": [user],
#   "sambaPwdLastSet": ["0"],
#   "sambaLogonTime": ["0"],
#   "sambaLogoffTime": ["2147483647"],
#   "sambaKickoffTime": ["2147483647"],
#   "sambaPwdCanChange": ["0"],
#   "sambaPwdMustChange": ["2147483647"],
#   "sambaAcctFlags": ["[UX]"],
#   "sambaLMPassword": ["XXX"],
#   "sambaNTPassword": ["XXX"],
#   "sambaLogonScript": ["%s.cmd" % user],
#   "sambaHomeDrive": ["H:"],

class Group(octofuss.PyTree):
    def __init__(self, data):
        super(Group, self).__init__(quote_plus(data.cn[0]), doc = "Group " + data.cn[0])
        self.data = data
        self.members = MemberList("members", data.cn[0])
        self.__dict__[":actions:"] = act.GroupActions(":actions:", self)
    def _get(self):
        return dict(
            cn = self.data.cn[0],
            gidNumber = int(self.data.gidNumber[0]),
#           members = [userDB.user(x).get("") for x in self.data["members"]]
        )

    cn = property(lambda x:x._name, doc="Group name")
    name = property(lambda x:x._name, doc="Group name")
    gidNumber = property(lambda x:x.data.gidNumber[0], doc="Numeric group ID")
