import unittest
import octofuss
import xmlrpclib
from urllib import quote_plus


SERVERURL = "http://localhost:13400/conf"
USERNAME = "test"
PASSWORD = "test"
PREFIX = "octofussd-test-"
MAXSHARES = 3

def _share(num):
	if num >= MAXSHARES:
		raise ValueError, "Attempt to generate share name #%d that will not be cleaned" % num
	return PREFIX+"share"+str(num)

class TestSamba(unittest.TestCase):
	def setUp(self):
		self.loginserver = xmlrpclib.ServerProxy(SERVERURL)
		self.apikey = self.loginserver.login(USERNAME, PASSWORD)
		self.tree = octofuss.xmlrpc.Client(SERVERURL, self.apikey)

		# Cleanup test shares
		for i in range(MAXSHARES):
			if self.tree.has("/samba/"+_share(i)):
				self.tree.delete("/samba/"+_share(i))

	def assertHas(self, path):
		if not self.tree.has(path):
			self.fail("Tree does not have path " + path)

	def assertNotHas(self, path):
		if self.tree.has(path):
			self.fail("Tree does have path " + path)

	def assertInList(self, name, path):
		for i in self.tree.list(path):
			if i == name:
				return
		self.fail("'%s' does not appear when listing path %s" % (name, path))

	def assertNotInList(self, name, path):
		for i in self.tree.list(path):
			if i == name:
				self.fail("'%s' does appear when listing path %s" % (name, path))

	def assertInTree(self, path):
		self.assertHas(path)
		parent, leaf = path.rsplit("/", 1)
		self.assertInList(leaf, parent)

	def assertNotInTree(self, path):
		self.assertNotHas(path)
		parent, leaf = path.rsplit("/", 1)
		self.assertNotInList(leaf, parent)

	def testCreateShare(self):
		share = _share(0)
		sharepath = "/samba/" + share
		self.assertNotHas(sharepath)
		self.tree.create(sharepath, {})
		self.assertInTree(sharepath)

	def testDeleteShare(self):
		share = _share(0)
		sharepath = "/samba/" + share
		self.assertNotHas(sharepath)
		self.tree.create(sharepath, {})
		self.assertInTree(sharepath)
		self.tree.delete(sharepath)
		self.assertNotInTree(sharepath)
